package gauntlet;

import java.util.*;
import com.golden.gamedev.object.Sprite;

public class Grid implements Iterable<Item>
{
    private Item[][][] data;
    private List<Item> order;
    private Queue<Item> add, rem;
    private boolean sort;

    public int balls, cursorX, cursorY;
    public boolean offscreenUpdates = true;
    public boolean animateTeleports = false;

    public Grid(int width, int length, int height)
    {
        data = new Item[width][length][height];
        order = new LinkedList<Item>();
        add = new LinkedList<Item>();
        rem = new LinkedList<Item>();
        sort = false;
    }

    //
    // get()
    //
    // get the item at location (x,y,z).
    //
    public Item get(int x, int y, int z)
    {
        return data[x][y][z];
    }

    //
    // set()
    //
    // set location (x,y,z) to have Item i. About
    // 99% of this method is devoted to servicing
    // the *iterator*.
    //
    public void set(int x, int y, int z, Item i)
    {
        //get the item currently at (x,y,z)
        //"d" means the item to be deleted
        Item d = data[x][y][z];

        //if we're setting the same item to the
        //same place, we're done
        if (d==i) return;

        //else if there is an item already there
        if (d != null)
        {
            if (d.holds <= 1)
            {
                d.holds = 0;
                rem.add(d);
            }
            else
            {
                d.rank *= 2;
                d.rank -= (x+y+z);
                d.holds--;
            }
        }

        if (i != null)
        {
            if (i.holds <= 0)
            {
                i.rank = (x+y+z);
                i.holds = 1;
                add.add(i);
            }
            else
            {
                i.rank += (x+y+z);
                i.rank /= 2;
                i.holds++;
            }
        }

        //set (x,y,z) position to have item i
        data[x][y][z] = i;

        //do a sort next time we iterate
        sort = true;
    }

    // Returns the highest item at the given coordinates.
    public Item getTop(int x, int y)
    {
        return getBelow(x, y, height());
    }

    // Returns the highest item below the given coordinates.
    public Item getBelow(int x, int y, int z)
    {
        do{ z--; }while(z>0 && data[x][y][z]==null);
        return data[x][y][z];
    }

    // Returns the force applied to objects at (x,y,z).
    public Direction getForce(int x, int y, int z)
    {
        //if the location in question is not on the floor
        if (z > 0)
        {
            //get the item below it
            Item i = data[x][y][z-1];

            //if there *is* an item below it,
            //return its force. By default, the
            //force() returns a Direction.STOP, but
            //this can be overridden. Direction.STOP
            //means there is *no* force. Turboblocks
            //have direction NSEW for force. Regular
            //blocks have no force.
            if (i!=null) return i.force();
        }
        return Direction.STOP; 
    }

    // Returns the adjacent elevation in the specified direction.
    public int getSurface(Item i, Direction d)
    {
        int x = i.x + d.x;
        int y = i.y + d.y;
        int z = i.z + d.z;

        if(i.z>0)
        {
            Item u = data[i.x][i.y][i.z-1];
            if(u!=null) z += u.slope(d);
        }

        while(z<height() && data[x][y][z]!=null) z++;
        return z;
    }

    public Iterator<Item> iterator()
    {
        if (sort)
        {
            order.addAll(add);
            order.removeAll(rem);
            add.clear();
            rem.clear();
            Collections.sort(order);
            sort = false;
        }
        return order.iterator();
    }

    public int width()
    {
        return data.length;
    }

    public int length()
    {
        return data[0].length;
    }

    public int height()
    {
        return data[0][0].length;
    }
}
