package gauntlet;

// Base class for all items in the grid.

public abstract class Item implements Comparable<Item>
{
    protected Grid g;

    public int x,y,z; // Position
    public Direction motion;
    //hasAttributes is true if setAttributes has been called
    public boolean mobile, hasAttributes;
    public int offsetX, offsetY; // Pixel offset for image

    public String imageFile;
    public Object sprite;
        public boolean removed = false;

    // Used to determine rendering position
    int holds = 0;
    double rank = 0;

    public Item(Grid g, int x, int y, int z)
    {
        this.g = g;
        this.x = x;
        this.y = y;
        this.z = z;
        motion = Direction.STOP;
        mobile = false;
        offsetX = offsetY = 0;
        imageFile = null;
        sprite = null;
    }

    // Call only from move().
    // Checks for gravity and collisons.
    // Returns true if the value of motion is changed.
    protected boolean baseMove()
    {
        Item i = (z>0) ? g.get(x,y,z-1) : null;

        //if our item is on top and mobile, or if
        //it's not on the floor and there's nothing underneath
        //
        if((z==0 && mobile) || (z>0 && i==null))
        {
            //start the item moving down
            motion = Direction.DOWN;
            return true;
        }

        //if we're not moving, we're done
        if(motion==Direction.STOP) return false;

        //check internal boundaries
        if(i!=null && !i.hasExit(motion))
        {
            motion = Direction.STOP;
            return true;
        }

        //if we're within the grid's bounds
        if(inBounds())
        {
            //get the topmost item in the direction we're headed
            i = g.getTop(x+motion.x, y+motion.y);

            //if there is such an item, and we *can't* move on top
            //of it
            if(i!=null && !i.canReceive(this))
            {
                //go no further
                motion = Direction.STOP;
                return true;
            }
        }
        return false;
    }

    // Checks whether the given item can be moved onto this.
    public boolean canReceive(Item i)
    {
        return (i.z > z);
    }

    public final int compareTo(Item i)
    {
        if(rank  < i.rank)  return -1;
        if(rank  > i.rank)  return 1;
        if(holds < i.holds) return -1;
        if(holds > i.holds) return 1;
        return 0;
    }

    // Returns the applied force, if any
    public Direction force()
    {
        return Direction.STOP;
    }

    // Returns true if an item can leave from above
    // this position in the specified direction.
    public boolean hasExit(Direction d)
    {
        return (d != Direction.DOWN);
    }

    // Returns false if the next motion will put this off the edge.
    public final boolean inBounds()
    {
        switch(motion)
        {
            case NORTH:
                if(y==0) return false;
                break;
            case SOUTH:
                if(y+1 == g.length()) return false;
                break;
            case EAST:
                if(x+1 == g.width()) return false;
                break;
            case WEST:
                if(x==0) return false;
                break;
            case UP:
                if(z+1 == g.height()) motion = Direction.STOP;
                break;
            case DOWN:
                if(z==0) return false;
        }
        return true;
    }

    // Called at the beginning of each animation cycle to determine motion.
    public Direction move()
    {
        //if we're not mobile, of course we can't move
        if(!mobile) return Direction.STOP;

        Direction d = motion;
        motion = g.getForce(x,y,z);
        if(motion==d || motion==null || motion==Direction.STOP || baseMove())
        {
            motion = d;
            baseMove();
        }
        d = motion;
        if(d!=Direction.STOP && inBounds())
        {
            motion = g.getForce(x+motion.x, y+motion.y, z);
            if(motion==null) motion = d;
        }
        return d;
    }

    // Performs actions upon receiving an item.
    // Returns true if the item is stopped or removed.
    public boolean receive(Item i)
    {
        return false;
    }

    // Called when the user selects this object.
    // Returns false only when this object is not selectable.
    public boolean select()
    {
        return mobile && (motion==Direction.STOP);
    }

    public void setAttributes(String data)
    {
        /* Do nothing */
    }

    // Called when the level editor needs to find out if an Item has any
    // attributes set.  Will return a string that can be used to re-create
    // those attributes via setAttributes.
    public String getAttributeString()
    {
        return "";  //nothing by default
    }

    // Used to determine the elevation of the next surface
    // in the given direction.  By default, always flat.
    public int slope(Direction d)
    {
        return 0;
    }

    //Each Item needs to print its type for the unparser.
    public abstract String toString();
}
